package com.supervision.pdfqaserver.dao;

import com.supervision.pdfqaserver.dto.neo4j.NodeData;
import com.supervision.pdfqaserver.dto.neo4j.RelationObject;
import com.supervision.pdfqaserver.dto.neo4j.RelationshipData;
import lombok.RequiredArgsConstructor;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Result;
import org.neo4j.driver.Session;
import org.neo4j.driver.types.Node;
import org.neo4j.driver.types.Relationship;
import org.springframework.stereotype.Repository;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;
import static org.neo4j.driver.Values.parameters;

@Repository
@RequiredArgsConstructor
public class Neo4jRepository {
    /**
     * Neo4j 驱动
     */
    private final Driver driver;

    /**
     * 执行传入的 Cypher 语句并返回 RelationObject 列表
     *
     * @param cypher  完整的预生成 Cypher 语句，比如：
     *                "MATCH (a)-[r:REL_TYPE]->(b) RETURN a AS startNode, r AS rel, b AS endNode"
     * @param params  语句参数（可为空）
     */
    public List<RelationObject> execute(String cypher, Map<String, Object> params) {
        try (Session session = driver.session()) {
            return session.executeRead(tx -> {
                Result result = tx.run(cypher, params == null ? Collections.emptyMap() : params);
                List<RelationObject> list = new ArrayList<>();
                while (result.hasNext()) {
                    org.neo4j.driver.Record record = result.next();
                    // 从 Record 中取出三部分
                    if (record.get("c") == null || record.get("r") == null || record.get("t") == null) {
                        continue;
                    }
                    Node a = record.get("c").asNode();
                    Relationship r = record.get("r").asRelationship();
                    Node b = record.get("t").asNode();

                    // 转成我们的 DTO
                    NodeData start = mapNode(a);
                    RelationshipData rel = mapRel(r);
                    NodeData end = mapNode(b);

                    list.add(new RelationObject(start, rel, end));
                }
                return list;
            });
        }
    }


    /**
     * 创建或更新实体节点
     * 根据唯一键（uniqueKey）来判断节点是否存在
     * note: properties中需要包含唯一键的值
     * @param label label
     * @param uniqueKey 唯一键
     * @param properties 属性
     * @return 节点ID列表
     */
    public List<Long> saveOrUpdateEntityNode(String label,String uniqueKey,Map<String, Object> properties) {

        try (Session session = driver.session()) {
            // MERGE语句确保唯一性
            String query = String.format(
                    "MERGE (n:%s {%s: $uniqueValue}) " +
                            "SET n += $properties " +
                            "RETURN id(n)  as id ",
                    label, uniqueKey);

            Result result = session.run(query, parameters("uniqueValue", properties.get(uniqueKey), "properties", properties));
            return result.list().stream().map(record -> record.get("id").asLong()).collect(Collectors.toList());
        }
    }

    /**
     * 创建或更新关系
     * @param sourceId 头节点ID
     * @param targetId 尾节点ID
     * @param relationType 关系类型
     * @param leftDirection 方向左
     * @param rightDirection 方向右
     * @param properties 关系属性
     * @return
     */
    public List<Long> saveOrUpdateRelation(Long sourceId,Long targetId,String relationType,
                                            boolean leftDirection,boolean rightDirection,Map<String, Object> properties) {

        try (Session session = driver.session()) {
            // MERGE关系
            String query = "MATCH (a) WHERE id(a) = $sourceId " +
                    "MATCH (b) WHERE id(b) = $targetId " +
                    "MERGE (a)%s-[r:" + relationType + "]-%s(b) " +
                    "SET r += $properties " +
                    "RETURN id(r) as id";

            String format = String.format(query,
                    leftDirection ? "<" : "",
                    rightDirection ? ">" : "");

            Result result = session.run(format,
                    parameters("sourceId", sourceId, "targetId", targetId, "properties", properties));

            return result.stream().map(record -> record.get("id").asLong()).collect(Collectors.toList());
        }
    }


    private NodeData mapNode(Node node) {
        return new NodeData(
                node.id(),
                StreamSupport.stream(node.labels().spliterator(), false).collect(Collectors.toList()),
                node.asMap()
        );
    }

    private RelationshipData mapRel(Relationship r) {
        return new RelationshipData(
                r.id(),
                r.type(),
                r.asMap()
        );
    }
}
